package com.hw.wms.service.impl;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

import com.hw.common.core.constant.SecurityConstants;
import com.hw.common.core.constant.WmsConstants;
import com.hw.common.core.domain.R;
import com.hw.common.core.exception.ServiceException;
import com.hw.common.security.utils.SecurityUtils;
import com.hw.mes.api.RemoteMesService;
import com.hw.mes.api.domain.MesBaseMaterialInfo;
import com.hw.mes.api.domain.vo.MesBaseMaterialInfoVo;
import com.hw.wms.domain.WmsRawStock;
import com.hw.wms.domain.vo.WmsRawOutstockAuditVo;
import com.hw.wms.domain.vo.WmsRawOutstockConfirmVo;
import com.hw.wms.mapper.WmsRawOutstockDetailMapper;
import com.hw.wms.mapper.WmsRawStockMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;

import com.hw.common.core.utils.StringUtils;
import org.springframework.transaction.annotation.Transactional;
import com.hw.wms.domain.WmsRawOutstockDetail;
import com.hw.wms.mapper.WmsRawOutstockMapper;
import com.hw.wms.domain.WmsRawOutstock;
import com.hw.wms.service.IWmsRawOutstockService;

/**
 * 原材料出库记录Service业务层处理
 *
 * @author xs
 * @date 2023-12-20
 */
@Service
public class WmsRawOutstockServiceImpl implements IWmsRawOutstockService {
    @Autowired
    private WmsRawOutstockMapper wmsRawOutstockMapper;

    @Autowired
    private WmsRawOutstockDetailMapper wmsRawOutstockDetailMapper;

    @Autowired
    private WmsRawStockMapper wmsRawStockMapper;

    @Autowired
    private RemoteMesService remoteMesService;

    /**
     * 查询原材料出库记录
     *
     * @param rawOutstockId 原材料出库记录主键
     * @return 原材料出库记录
     */
    @Override
    public WmsRawOutstock selectWmsRawOutstockByRawOutstockId(Long rawOutstockId) {
        WmsRawOutstock wmsRawOutstock = wmsRawOutstockMapper.selectWmsRawOutstockByRawOutstockId(rawOutstockId);
        List<WmsRawOutstockDetail> wmsRawOutstockDetailList = wmsRawOutstock.getWmsRawOutstockDetailList();
        Long[] materialIds = wmsRawOutstockDetailList.stream().map(o -> o.getMaterialId()).toArray(Long[]::new);
        MesBaseMaterialInfoVo mesBaseMaterialInfoVo = new MesBaseMaterialInfoVo();
        mesBaseMaterialInfoVo.setMaterialIds(materialIds);
        R<List<MesBaseMaterialInfo>> materialInfosR = remoteMesService.getMaterialsByMaterialIds(mesBaseMaterialInfoVo, SecurityConstants.INNER);
        if (materialInfosR != null) {
            List<MesBaseMaterialInfo> materialInfos = materialInfosR.getData();

            Map<Long, MesBaseMaterialInfo> MesBaseMaterialInfoMap = materialInfos.stream()
                    .collect(Collectors.toMap(MesBaseMaterialInfo::getMaterialId, mesBaseMaterialInfo -> mesBaseMaterialInfo));

            wmsRawOutstockDetailList.forEach(wmsRawOutstockDetail -> {
                MesBaseMaterialInfo materialInfo = MesBaseMaterialInfoMap.get(wmsRawOutstockDetail.getMaterialId());
                if (materialInfo != null) {
                    wmsRawOutstockDetail.setMaterialCode(materialInfo.getMaterialCode());
                    wmsRawOutstockDetail.setMaterialName(materialInfo.getMaterialName());
                }
            });
        }

        return wmsRawOutstock;
    }

    /**
     * 查询原材料出库记录列表
     *
     * @param wmsRawOutstock 原材料出库记录
     * @return 原材料出库记录
     */
    @Override
    public List<WmsRawOutstock> selectWmsRawOutstockList(WmsRawOutstock wmsRawOutstock) {
        return wmsRawOutstockMapper.selectWmsRawOutstockList(wmsRawOutstock);
    }

    /**
     * 新增原材料出库记录
     *
     * @param wmsRawOutstock 原材料出库记录
     * @return 结果
     */
    @Transactional
    @Override
    public int insertWmsRawOutstock(WmsRawOutstock wmsRawOutstock) {
        int rows = wmsRawOutstockMapper.insertWmsRawOutstock(wmsRawOutstock);
        insertWmsRawOutstockDetail(wmsRawOutstock);
        return rows;
    }

    /**
     * 修改原材料出库记录
     *
     * @param wmsRawOutstock 原材料出库记录
     * @return 结果
     */
    @Transactional
    @Override
    public int updateWmsRawOutstock(WmsRawOutstock wmsRawOutstock) {
        wmsRawOutstockMapper.deleteWmsRawOutstockDetailByRawOutstockId(wmsRawOutstock.getRawOutstockId());
        insertWmsRawOutstockDetail(wmsRawOutstock);
        return wmsRawOutstockMapper.updateWmsRawOutstock(wmsRawOutstock);
    }

    /**
     * 批量删除原材料出库记录
     *
     * @param rawOutstockIds 需要删除的原材料出库记录主键
     * @return 结果
     */
    @Transactional
    @Override
    public int deleteWmsRawOutstockByRawOutstockIds(Long[] rawOutstockIds) {
        wmsRawOutstockMapper.deleteWmsRawOutstockDetailByRawOutstockIds(rawOutstockIds);
        return wmsRawOutstockMapper.deleteWmsRawOutstockByRawOutstockIds(rawOutstockIds);
    }

    /**
     * 删除原材料出库记录信息
     *
     * @param rawOutstockId 原材料出库记录主键
     * @return 结果
     */
    @Transactional
    @Override
    public int deleteWmsRawOutstockByRawOutstockId(Long rawOutstockId) {
        wmsRawOutstockMapper.deleteWmsRawOutstockDetailByRawOutstockId(rawOutstockId);
        return wmsRawOutstockMapper.deleteWmsRawOutstockByRawOutstockId(rawOutstockId);
    }

    /**
     * 新增原材料出库记录明细信息
     *
     * @param wmsRawOutstock 原材料出库记录对象
     */
    public void insertWmsRawOutstockDetail(WmsRawOutstock wmsRawOutstock) {
        List<WmsRawOutstockDetail> wmsRawOutstockDetailList = wmsRawOutstock.getWmsRawOutstockDetailList();
        Long rawOutstockId = wmsRawOutstock.getRawOutstockId();
        if (StringUtils.isNotNull(wmsRawOutstockDetailList)) {
            List<WmsRawOutstockDetail> list = new ArrayList<WmsRawOutstockDetail>();
            for (WmsRawOutstockDetail wmsRawOutstockDetail : wmsRawOutstockDetailList) {
                wmsRawOutstockDetail.setRawOutstockId(rawOutstockId);
                list.add(wmsRawOutstockDetail);
            }
            if (list.size() > 0) {
                wmsRawOutstockMapper.batchWmsRawOutstockDetail(list);
            }
        }
    }


    /**
     * 审核原材料出库记录
     *
     * @param wmsRawOutstockAuditVo
     * @return
     */
    @Transactional
    @Override
    public int auditWmsRawOutstock(WmsRawOutstockAuditVo wmsRawOutstockAuditVo) {
        String auditStatus = wmsRawOutstockAuditVo.getAuditStatus();
        String auditReason = wmsRawOutstockAuditVo.getAuditReason();
        Date currentDate = new Date();
        String userName = SecurityUtils.getUsername();
        WmsRawOutstock wmsRawOutstock = wmsRawOutstockMapper.selectWmsRawOutstockByRawOutstockId(wmsRawOutstockAuditVo.getRawOutstockId());
        wmsRawOutstock.setAuditStatus(auditStatus);
        wmsRawOutstock.setAuditReason(auditReason);
        wmsRawOutstock.setAuditBy(SecurityUtils.getUsername());
        wmsRawOutstock.setAuditDate(new Date());

        //审核不通过，需要有审核意见，并且需要更新库位占用数量。
        if (auditStatus.equalsIgnoreCase(WmsConstants.RAW_OUTSTOCK_AUDIT_STATUS_FAIL)) {
            if (StringUtils.isEmpty(auditReason)) {
                throw new ServiceException("请输入审核意见");
            }
            List<WmsRawOutstockDetail> wmsRawOutstockDetails = wmsRawOutstock.getWmsRawOutstockDetailList();
            wmsRawOutstockDetails.forEach(wmsRawOutstockDetail -> {
                String locationCode = wmsRawOutstockDetail.getLocationCode();
                String instockBatch = wmsRawOutstockDetail.getInstockBatch();
                BigDecimal planAmount = wmsRawOutstockDetail.getPlanAmount();
                WmsRawStock wmsRawStock = wmsRawStockMapper.selectRawStockByLocationAndBatch(locationCode, instockBatch);
                if (wmsRawStock != null) {
                    BigDecimal occupyAmount = wmsRawStock.getOccupyAmount() == null ?
                            BigDecimal.ZERO : wmsRawStock.getOccupyAmount();
                    if (occupyAmount.compareTo(planAmount) >= 0) {
                        wmsRawStock.setOccupyAmount(occupyAmount.subtract(planAmount));
                        wmsRawStock.setUpdateBy(userName);
                        wmsRawStock.setUpdateDate(currentDate);
                    } else {
                        wmsRawStock.setOccupyAmount(BigDecimal.ZERO);
                        wmsRawStock.setUpdateBy(userName);
                        wmsRawStock.setUpdateDate(currentDate);
                    }
                    wmsRawStockMapper.updateWmsRawStock(wmsRawStock);
                }
            });

        }

        return wmsRawOutstockMapper.updateWmsRawOutstock(wmsRawOutstock);

    }

    /**
     * 原材料出库记录明细确认
     *
     * @param wmsRawOutstockConfirmVo
     * @return
     */
    @Transactional
    @Override
    public int confirmWmsRawOutstock(WmsRawOutstockConfirmVo wmsRawOutstockConfirmVo) {
        Long rawOutstockDetailId = wmsRawOutstockConfirmVo.getRawOutstockDetailId();
        String locationCode = wmsRawOutstockConfirmVo.getLocationCode();
        String instockBatch = wmsRawOutstockConfirmVo.getInstockBatch();
        BigDecimal outstockAmount = wmsRawOutstockConfirmVo.getOutstockAmount();
        //判断是否有此出库任务,人工出库的同一个出库记录，同一个库位、同一个物料对应一个明细（agv的有可能是多个）
        WmsRawOutstockDetail queryRawOutstockDetail = new WmsRawOutstockDetail();
        queryRawOutstockDetail.setLocationCode(locationCode);
        queryRawOutstockDetail.setInstockBatch(instockBatch);
        List<WmsRawOutstockDetail> wmsRawOutstockDetails = wmsRawOutstockDetailMapper.selectWmsRawOutstockDetailList(queryRawOutstockDetail);
        WmsRawOutstockDetail wmsRawOutstockDetail = new WmsRawOutstockDetail();
        if (wmsRawOutstockDetails == null || wmsRawOutstockDetails.isEmpty()) {
            throw new ServiceException("没有此出库任务");
        } else if (wmsRawOutstockDetails.size() > 1) {
            if (rawOutstockDetailId == null) {
                throw new ServiceException("请选择具体出库任务后点击确认");
            } else {
                wmsRawOutstockDetail = wmsRawOutstockDetails.stream().filter(wro -> wro.getRawOutstockDetailId().equals(rawOutstockDetailId)).findFirst().get();
            }
        } else {
            wmsRawOutstockDetail = wmsRawOutstockDetails.get(0);
        }

        //判断质检状态
        if (wmsRawOutstockDetail.getQualityStatus() != null
                && !wmsRawOutstockDetail.getQualityStatus().equals(WmsConstants.RAW_OUTSTOCK_DETAIL_QUALITY_STATUS_PASS)) {
            throw new ServiceException("质检通过才能出库");
        }

        // 判断本次出库数量+已出库数量是否大于计划出库数量
        BigDecimal hadOutstockAmount = wmsRawOutstockDetail.getOutstockAmount() == null ? BigDecimal.ZERO : wmsRawOutstockDetail.getOutstockAmount();
        BigDecimal planOutstockAmount = wmsRawOutstockDetail.getPlanAmount();

        BigDecimal updatedOutstockAmount = hadOutstockAmount.add(outstockAmount);
        if (updatedOutstockAmount.compareTo(planOutstockAmount) > 0) {
            throw new ServiceException("出库数量不能大于" + planOutstockAmount);
        }

        wmsRawOutstockDetail.setOutstockAmount(updatedOutstockAmount);
        String executeStatus = "";
        if (updatedOutstockAmount.compareTo(planOutstockAmount) == 0) {
            executeStatus = WmsConstants.RAW_INSTOCK_EXECUTE_STATUS_FINISH;
            //更新出库记录执行状态
            this.updateRawOutstockDetailExecuteStatus(wmsRawOutstockDetail.getRawOutstockId(),
                    wmsRawOutstockDetail.getRawOutstockDetailId());
        } else {
            executeStatus = WmsConstants.RAW_INSTOCK_EXECUTE_STATUS_EXECUTING;
        }

        Date currentDate = new Date();
        String userName = SecurityUtils.getUsername();
        wmsRawOutstockDetail.setExecuteStatus(executeStatus);
        if (wmsRawOutstockDetail.getOutstockTime() == null) {
            wmsRawOutstockDetail.setOutstockTime(currentDate);
        } else {
            wmsRawOutstockDetail.setUpdateDate(currentDate);
        }
        if (wmsRawOutstockDetail.getOutstockPerson() == null) {
            wmsRawOutstockDetail.setOutstockPerson(userName);
        } else {
            wmsRawOutstockDetail.setUpdateBy(userName);
        }

        //更新原材料库存占用数量和总数量
        WmsRawStock wmsRawStock = wmsRawStockMapper.selectRawStockByLocationAndBatch(locationCode, instockBatch);
        if (wmsRawStock == null) {
            throw new ServiceException("数据异常");
        } else {
            BigDecimal totalAmount = wmsRawStock.getTotalAmount() == null ? BigDecimal.ZERO : wmsRawStock.getTotalAmount();
            BigDecimal occupyAmount = wmsRawStock.getOccupyAmount() == null ? BigDecimal.ZERO : wmsRawStock.getOccupyAmount();
            BigDecimal updatedOccupyAmount = occupyAmount.subtract(outstockAmount);
            BigDecimal updatedTotalAmount = totalAmount.subtract(outstockAmount);
            updatedOccupyAmount = updatedOccupyAmount.compareTo(BigDecimal.ZERO) >= 0 ? updatedOccupyAmount : BigDecimal.ZERO;
            updatedTotalAmount = updatedTotalAmount.compareTo(BigDecimal.ZERO) >= 0 ? updatedTotalAmount : BigDecimal.ZERO;
            wmsRawStock.setOccupyAmount(updatedOccupyAmount);
            wmsRawStock.setTotalAmount(updatedTotalAmount);
            wmsRawStock.setUpdateDate(currentDate);
            wmsRawStock.setUpdateBy(userName);
            wmsRawStockMapper.updateWmsRawStock(wmsRawStock);
        }

        wmsRawOutstockDetailMapper.updateWmsRawOutstockDetail(wmsRawOutstockDetail);

        //todo 同步erp
        return 1;
    }

    /**
     * 查询原材料出库记录明细列表，供手持出库使用
     *
     * @param wmsRawOutstockDetail 原材料出库记录明细
     * @return 原材料出库记录明细
     */
    @Override
    public List<WmsRawOutstockDetail> selectAuditPassRawOutstockDetails(WmsRawOutstockDetail wmsRawOutstockDetail) {
        return wmsRawOutstockDetailMapper.selectAuditPassRawOutstockDetails(wmsRawOutstockDetail);
    }


    /**
     * 更新原材料出库记录的执行状态
     *
     * @param rawOutstockId
     * @param rawOutStockDetailId
     */
    private void updateRawOutstockDetailExecuteStatus(Long rawOutstockId, Long rawOutStockDetailId) {
        WmsRawOutstockDetail queryRawOutstockDetail = new WmsRawOutstockDetail();
        queryRawOutstockDetail.setRawOutstockId(rawOutstockId);
        List<WmsRawOutstockDetail> wmsRawOutstockDetails = wmsRawOutstockDetailMapper.selectWmsRawOutstockDetailList(queryRawOutstockDetail);

        boolean isExecuteFinish = true;
        for (WmsRawOutstockDetail wmsRawOutstockDetail : wmsRawOutstockDetails) {
            if (!wmsRawOutstockDetail.getRawOutstockDetailId().equals(rawOutStockDetailId)
                    && !wmsRawOutstockDetail.getExecuteStatus().equals(WmsConstants.RAW_OUTSTOCK_EXECUTE_STATUS_FINISH)) {
                isExecuteFinish = false;
            }
        }

        if (isExecuteFinish) {
            WmsRawOutstock wmsRawOutstock = wmsRawOutstockMapper.selectWmsRawOutstockByRawOutstockId(rawOutstockId);
            wmsRawOutstock.setExecuteStatus(WmsConstants.RAW_OUTSTOCK_EXECUTE_STATUS_FINISH);
            wmsRawOutstockMapper.updateWmsRawOutstock(wmsRawOutstock);
        }
    }


    /**
     * 申请领料
     *
     * @param wmsRawOutstock 原材料出库记录
     * @return 结果
     */
    @Transactional
    @Override
    public int applyRawOutstock(WmsRawOutstock wmsRawOutstock) {
        Long warehouseId = wmsRawOutstock.getWarehouseId();
        List<WmsRawOutstockDetail> rawOutstockDetails = wmsRawOutstock.getWmsRawOutstockDetailList();
        //如果同一个物料有多条记录的处理,提前更新，放在事务中
        //判断库位和可用数量（总数量减冻结数量和占用数量），按先进先出（按入库时间排序获取）
        String queryQualityStatus = WmsConstants.RWA_STOCK_QUALITY_STATUS_PASS;
        List<WmsRawOutstockDetail> toUpdatedRawOutstockDetails = new ArrayList<>();
        Date currentDate = new Date();
        String userName = SecurityUtils.getUsername();
        rawOutstockDetails.forEach(rawOutstockDetail -> {
            BigDecimal planAmount = rawOutstockDetail.getPlanAmount();
            BigDecimal realRawStock = BigDecimal.ZERO;//实际库存总量
            Long materialId = rawOutstockDetail.getMaterialId();
            WmsRawStock queryRawStock = new WmsRawStock();
            queryRawStock.setWarehouseId(warehouseId);
            queryRawStock.setMaterialId(materialId);
            queryRawStock.setQualityStatus(queryQualityStatus);
            List<WmsRawStock> wmsRawStocks = wmsRawStockMapper.selectWmsRawStocks4Apply(queryRawStock);

            if (wmsRawStocks != null && !wmsRawStocks.isEmpty()) {
                for (WmsRawStock wmsRawStock : wmsRawStocks) {
                    BigDecimal totalAmount = wmsRawStock.getTotalAmount();
                    BigDecimal frozenAmount = wmsRawStock.getFrozenAmount() == null ? BigDecimal.ZERO : wmsRawStock.getFrozenAmount();
                    BigDecimal occupyAmount = wmsRawStock.getOccupyAmount() == null ? BigDecimal.ZERO : wmsRawStock.getOccupyAmount();
                    BigDecimal availableAmount = totalAmount.subtract(frozenAmount).subtract(occupyAmount);
                    realRawStock = realRawStock.add(availableAmount);
                    //可出库数量大于0
                    if (availableAmount.compareTo(BigDecimal.ZERO) > 0) {
                        //如果申请数量大于此库位可出库的数量则继续进行下一库位判断
                        if (planAmount.compareTo(availableAmount) > 0) {
                            //库存信息更改
                            wmsRawStock.setOccupyAmount(occupyAmount.add(availableAmount));
                            wmsRawStock.setUpdateDate(currentDate);
                            wmsRawStock.setUpdateBy(userName);
                            wmsRawStockMapper.updateWmsRawStock(wmsRawStock);

                            //原材料出库记录明细
                            WmsRawOutstockDetail wmsRawOutstockDetail = getWmsRawOutstockDetail(wmsRawStock, materialId,
                                    availableAmount, currentDate, userName);
                            toUpdatedRawOutstockDetails.add(wmsRawOutstockDetail);

                            planAmount = planAmount.subtract(availableAmount);
                            continue;
                        } else if (planAmount.compareTo(availableAmount) == 0) {//计划数量等于可出库数量
                            //库存信息更改
                            wmsRawStock.setOccupyAmount(occupyAmount.add(availableAmount));
                            wmsRawStock.setUpdateDate(currentDate);
                            wmsRawStock.setUpdateBy(userName);
                            wmsRawStockMapper.updateWmsRawStock(wmsRawStock);

                            //原材料出库记录明细
                            WmsRawOutstockDetail wmsRawOutstockDetail = getWmsRawOutstockDetail(wmsRawStock, materialId,
                                    availableAmount, currentDate, userName);
                            toUpdatedRawOutstockDetails.add(wmsRawOutstockDetail);
                            planAmount = BigDecimal.ZERO;
                            break;
                        } else if (planAmount.compareTo(availableAmount) < 0) {
                            //库存信息更改
                            wmsRawStock.setOccupyAmount(occupyAmount.add(planAmount));
                            wmsRawStock.setUpdateDate(currentDate);
                            wmsRawStock.setUpdateBy(userName);
                            wmsRawStockMapper.updateWmsRawStock(wmsRawStock);

                            //原材料出库记录明细
                            WmsRawOutstockDetail wmsRawOutstockDetail = getWmsRawOutstockDetail(wmsRawStock, materialId,
                                    planAmount, currentDate, userName);
                            toUpdatedRawOutstockDetails.add(wmsRawOutstockDetail);
                            planAmount = BigDecimal.ZERO;
                            break;
                        }
                    }

                }
            }

            if (planAmount.compareTo(BigDecimal.ZERO) > 0) {
                throw new ServiceException("物料[" + rawOutstockDetail.getMaterialName() + "]库存不足，只有" + realRawStock);
            }

        });


        //todo 审核状态默认为待审核，后续可根据系统参数判断保存是待审核还是直接审核通过
        wmsRawOutstock.setApplyBy(userName);
        wmsRawOutstock.setApplyDate(currentDate);
        wmsRawOutstock.setAuditStatus(WmsConstants.RAW_OUTSTOCK_AUDIT_STATUS_TOAUDIT);
        wmsRawOutstock.setExecuteStatus(WmsConstants.RAW_OUTSTOCK_EXECUTE_STATUS_TO_EXECUTUTE);

        int rows = wmsRawOutstockMapper.insertWmsRawOutstock(wmsRawOutstock);
        wmsRawOutstock.setWmsRawOutstockDetailList(toUpdatedRawOutstockDetails);
        insertWmsRawOutstockDetail(wmsRawOutstock);

        return rows;
    }


    /**
     * 获取待保存的原材料出库记录明细
     *
     * @param wmsRawStock
     * @param materialId
     * @param availableAmount
     * @param currentDate
     * @param userName
     * @return
     */
    private WmsRawOutstockDetail getWmsRawOutstockDetail(WmsRawStock wmsRawStock, Long materialId,
                                                         BigDecimal availableAmount, Date currentDate, String userName) {
        WmsRawOutstockDetail wmsRawOutstockDetail = new WmsRawOutstockDetail();
        wmsRawOutstockDetail.setLocationCode(wmsRawStock.getLocationCode());
        wmsRawOutstockDetail.setMaterialId(materialId);
        wmsRawOutstockDetail.setInstockBatch(wmsRawStock.getInstockBatch());
        wmsRawOutstockDetail.setPlanAmount(availableAmount);
        wmsRawOutstockDetail.setExecuteStatus(WmsConstants.RAW_OUTSTOCK_EXECUTE_STATUS_TO_EXECUTUTE);
        wmsRawOutstockDetail.setOutstockWay(WmsConstants.RAW_OUTSTOCK_OUTSTOCK_WAY_MANUAL);
        wmsRawOutstockDetail.setQualityStatus(wmsRawStock.getQualityStatus());
        wmsRawOutstockDetail.setCreateDate(currentDate);
        wmsRawOutstockDetail.setCreateBy(userName);

        return wmsRawOutstockDetail;
    }


}
