package com.hw.wms.service.impl;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

import com.hw.common.core.constant.SecurityConstants;
import com.hw.common.core.constant.WmsConstants;
import com.hw.common.core.domain.R;
import com.hw.common.core.exception.ServiceException;
import com.hw.common.security.utils.SecurityUtils;
import com.hw.mes.api.RemoteMesService;
import com.hw.mes.api.domain.MesBaseMaterialInfo;
import com.hw.mes.api.domain.vo.MesBaseMaterialInfoVo;
import com.hw.wms.domain.*;
import com.hw.wms.domain.vo.WmsRawOutstockAuditVo;
import com.hw.wms.domain.vo.WmsRawOutstockConfirmVo;
import com.hw.wms.domain.vo.WmsRawReturnAuditVo;
import com.hw.wms.domain.vo.WmsRawReturnConfirmVo;
import com.hw.wms.mapper.WmsRawReturnDetailMapper;
import com.hw.wms.mapper.WmsRawStockMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Map;
import java.util.stream.Collectors;

import com.hw.common.core.utils.StringUtils;
import org.springframework.transaction.annotation.Transactional;
import com.hw.wms.mapper.WmsRawReturnMapper;
import com.hw.wms.service.IWmsRawReturnService;

import javax.annotation.Resource;

/**
 * 原材料退库记录Service业务层处理
 *
 * @author xs
 * @date 2023-12-20
 */
@Service
public class WmsRawReturnServiceImpl implements IWmsRawReturnService {
    @Autowired
    private WmsRawReturnMapper wmsRawReturnMapper;

    @Autowired
    private WmsRawReturnDetailMapper wmsRawReturnDetailMapper;

    @Autowired
    private WmsRawStockMapper wmsRawStockMapper;

    @Resource
    private RemoteMesService remoteMesService;

    /**
     * 查询原材料退库记录
     *
     * @param rawReturnId 原材料退库记录主键
     * @return 原材料退库记录
     */
    @Override
    public WmsRawReturn selectWmsRawReturnByRawReturnId(Long rawReturnId) {
        WmsRawReturn rawReturn = wmsRawReturnMapper.selectWmsRawReturnByRawReturnId(rawReturnId);
        List<WmsRawReturnDetail> wmsRawReturnDetailList = rawReturn.getWmsRawReturnDetailList();
        Long[] materialIds = wmsRawReturnDetailList.stream().map(o -> o.getMaterialId()).toArray(Long[]::new);
        MesBaseMaterialInfoVo mesBaseMaterialInfoVo = new MesBaseMaterialInfoVo();
        mesBaseMaterialInfoVo.setMaterialIds(materialIds);
        R<List<MesBaseMaterialInfo>> materialInfosR = remoteMesService.getMaterialsByMaterialIds(mesBaseMaterialInfoVo, SecurityConstants.INNER);
        if (materialInfosR != null) {
            List<MesBaseMaterialInfo> materialInfos = materialInfosR.getData();

            Map<Long, MesBaseMaterialInfo> MesBaseMaterialInfoMap = materialInfos.stream()
                    .collect(Collectors.toMap(MesBaseMaterialInfo::getMaterialId, mesBaseMaterialInfo -> mesBaseMaterialInfo));

            wmsRawReturnDetailList.forEach(wmsRawReturnDetail -> {
                MesBaseMaterialInfo materialInfo = MesBaseMaterialInfoMap.get(wmsRawReturnDetail.getMaterialId());
                if (materialInfo != null) {
                    wmsRawReturnDetail.setMaterialCode(materialInfo.getMaterialCode());
                    wmsRawReturnDetail.setMaterialName(materialInfo.getMaterialName());
                }
            });
        }

        return rawReturn;

    }

    /**
     * 查询原材料退库记录列表
     *
     * @param wmsRawReturn 原材料退库记录
     * @return 原材料退库记录
     */
    @Override
    public List<WmsRawReturn> selectWmsRawReturnList(WmsRawReturn wmsRawReturn) {
        return wmsRawReturnMapper.selectWmsRawReturnList(wmsRawReturn);
    }

    /**
     * 新增原材料退库记录
     *
     * @param wmsRawReturn 原材料退库记录
     * @return 结果
     */
    @Transactional
    @Override
    public int insertWmsRawReturn(WmsRawReturn wmsRawReturn) {
        int rows = wmsRawReturnMapper.insertWmsRawReturn(wmsRawReturn);
        insertWmsRawReturnDetail(wmsRawReturn);
        return rows;
    }

    /**
     * 修改原材料退库记录
     *
     * @param wmsRawReturn 原材料退库记录
     * @return 结果
     */
    @Transactional
    @Override
    public int updateWmsRawReturn(WmsRawReturn wmsRawReturn) {
        wmsRawReturnMapper.deleteWmsRawReturnDetailByRawReturnId(wmsRawReturn.getRawReturnId());
        insertWmsRawReturnDetail(wmsRawReturn);
        return wmsRawReturnMapper.updateWmsRawReturn(wmsRawReturn);
    }

    /**
     * 批量删除原材料退库记录
     *
     * @param rawReturnIds 需要删除的原材料退库记录主键
     * @return 结果
     */
    @Transactional
    @Override
    public int deleteWmsRawReturnByRawReturnIds(Long[] rawReturnIds) {
        wmsRawReturnMapper.deleteWmsRawReturnDetailByRawReturnIds(rawReturnIds);
        return wmsRawReturnMapper.deleteWmsRawReturnByRawReturnIds(rawReturnIds);
    }

    /**
     * 删除原材料退库记录信息
     *
     * @param rawReturnId 原材料退库记录主键
     * @return 结果
     */
    @Transactional
    @Override
    public int deleteWmsRawReturnByRawReturnId(Long rawReturnId) {
        wmsRawReturnMapper.deleteWmsRawReturnDetailByRawReturnId(rawReturnId);
        return wmsRawReturnMapper.deleteWmsRawReturnByRawReturnId(rawReturnId);
    }

    /**
     * 新增原材料退库记录明细信息
     *
     * @param wmsRawReturn 原材料退库记录对象
     */
    public void insertWmsRawReturnDetail(WmsRawReturn wmsRawReturn) {
        List<WmsRawReturnDetail> wmsRawReturnDetailList = wmsRawReturn.getWmsRawReturnDetailList();
        Long rawReturnId = wmsRawReturn.getRawReturnId();
        if (StringUtils.isNotNull(wmsRawReturnDetailList)) {
            List<WmsRawReturnDetail> list = new ArrayList<WmsRawReturnDetail>();
            for (WmsRawReturnDetail wmsRawReturnDetail : wmsRawReturnDetailList) {
                wmsRawReturnDetail.setRawReturnId(rawReturnId);
                list.add(wmsRawReturnDetail);
            }
            if (list.size() > 0) {
                wmsRawReturnMapper.batchWmsRawReturnDetail(list);
            }
        }
    }


    /**
     * 查询原材料退库记录列表,供手持退库审核使用
     *
     * @param wmsRawReturn 原材料退库记录
     * @return 原材料退库记录
     */
    @Override
    public List<WmsRawReturn> selectWmsRawReturnJoinList(WmsRawReturn wmsRawReturn) {
        return wmsRawReturnMapper.selectWmsRawReturnJoinList(wmsRawReturn);
    }


    /**
     * 审核原材料退库记录
     *
     * @param wmsRawReturnAuditVo
     */
    @Override
    public int auditWmsRawReturn(WmsRawReturnAuditVo wmsRawReturnAuditVo) {
        String auditStatus = wmsRawReturnAuditVo.getAuditStatus();
        String auditReason = wmsRawReturnAuditVo.getAuditReason();
        if (WmsConstants.RAW_RETURN_AUDIT_STATUS_FAIL.equals(auditStatus) && StringUtils.isEmpty(auditReason)) {
            throw new ServiceException("请输入审核意见");
        }

        WmsRawReturn wmsRawReturn = wmsRawReturnMapper.selectWmsRawReturnByRawReturnId(wmsRawReturnAuditVo.getRawReturnId());
        wmsRawReturn.setAuditStatus(auditStatus);
        wmsRawReturn.setAuditReason(auditReason);
        wmsRawReturn.setAuditBy(SecurityUtils.getUsername());
        wmsRawReturn.setAuditDate(new Date());
        return wmsRawReturnMapper.updateWmsRawReturn(wmsRawReturn);
    }

    /**
     * 原材料退库记录明细确认
     *
     * @param wmsRawReturnConfirmVo
     * @return
     */
    @Transactional
    @Override
    public int confirmWmsRawReturn(WmsRawReturnConfirmVo wmsRawReturnConfirmVo) {
        Long rawReturnDetailId = wmsRawReturnConfirmVo.getRawReturnDetailId();
        String locationCode = wmsRawReturnConfirmVo.getLocationCode();
        String instockBatch = wmsRawReturnConfirmVo.getInstockBatch();
        BigDecimal returnAmount = wmsRawReturnConfirmVo.getReturnAmount();
        //判断是否有此退库任务,人工退库的同一个退库记录，同一个库位、同一个物料对应一个明细（agv的有可能是多个）
        WmsRawReturnDetail queryRawReturnDetail = new WmsRawReturnDetail();
        queryRawReturnDetail.setLocationCode(locationCode);
        queryRawReturnDetail.setInstockBatch(instockBatch);
        List<WmsRawReturnDetail> wmsRawReturnDetails = wmsRawReturnDetailMapper.selectWmsRawReturnDetailList(queryRawReturnDetail);
        WmsRawReturnDetail wmsRawReturnDetail = new WmsRawReturnDetail();
        if (wmsRawReturnDetails == null || wmsRawReturnDetails.isEmpty()) {
            throw new ServiceException("没有此退库任务");
        } else if (wmsRawReturnDetails.size() > 1) {
            if (rawReturnDetailId == null) {
                throw new ServiceException("请选择具体退库任务后点击确认");
            } else {
                wmsRawReturnDetail = wmsRawReturnDetails.stream().filter(wrr -> wrr.getRawReturnDetailId().equals(rawReturnDetailId)).findFirst().get();
            }
        } else {
            wmsRawReturnDetail = wmsRawReturnDetails.get(0);
        }

        //判断质检状态
        if (wmsRawReturnDetail.getQualityStatus() != null
                && !wmsRawReturnDetail.getQualityStatus().equals(WmsConstants.RAW_RETURN_DETAIL_QUALITY_STATUS_PASS)) {
            throw new ServiceException("质检通过才能出库");
        }

        // 判断本次退库数量+已退库数量是否大于计划退库数量
        BigDecimal hadReturnAmount = wmsRawReturnDetail.getReturnAmount() == null ? BigDecimal.ZERO : wmsRawReturnDetail.getReturnAmount();
        BigDecimal planReturnAmount = wmsRawReturnDetail.getPlanAmount();

        BigDecimal updatedReturnAmount = hadReturnAmount.add(returnAmount);
        if (updatedReturnAmount.compareTo(planReturnAmount) > 0) {
            throw new ServiceException("退库数量不能大于" + planReturnAmount);
        }

        wmsRawReturnDetail.setReturnAmount(updatedReturnAmount);
        String executeStatus = "";
        if (updatedReturnAmount.compareTo(planReturnAmount) == 0) {
            executeStatus = WmsConstants.RAW_RETURN_EXECUTE_STATUS_FINISH;
            //更新退库记录执行状态
            this.updateRawReturnDetailExecuteStatus(wmsRawReturnDetail.getRawReturnId(),
                    wmsRawReturnDetail.getRawReturnDetailId());
        } else {
            executeStatus = WmsConstants.RAW_INSTOCK_EXECUTE_STATUS_EXECUTING;
        }

        Date currentDate = new Date();
        String userName = SecurityUtils.getUsername();
        wmsRawReturnDetail.setExecuteStatus(executeStatus);
        if (wmsRawReturnDetail.getReturnTime() == null) {
            wmsRawReturnDetail.setReturnTime(currentDate);
        } else {
            wmsRawReturnDetail.setUpdateDate(currentDate);
        }
        if (wmsRawReturnDetail.getReturnPerson() == null) {
            wmsRawReturnDetail.setReturnPerson(userName);
        } else {
            wmsRawReturnDetail.setUpdateBy(userName);
        }

        wmsRawReturnDetailMapper.updateWmsRawReturnDetail(wmsRawReturnDetail);

        //更新原材料库存总数量
        WmsRawStock wmsRawStock = wmsRawStockMapper.selectRawStockByLocationAndBatch(locationCode, instockBatch);
        if (wmsRawStock == null) {
            throw new ServiceException("数据异常");
        } else {
            BigDecimal totalAmount = wmsRawStock.getTotalAmount() == null ? BigDecimal.ZERO : wmsRawStock.getTotalAmount();
            BigDecimal updatedTotalAmount = totalAmount.add(returnAmount);
            wmsRawStock.setTotalAmount(updatedTotalAmount);
            wmsRawStock.setUpdateDate(currentDate);
            wmsRawStock.setUpdateBy(userName);
            wmsRawStockMapper.updateWmsRawStock(wmsRawStock);
        }

        //todo 同步erp
        return 1;
    }

    /**
     * 查询原材料退记录明细列表，供手持出库使用
     *
     * @param wmsRawReturnDetail 原材料退库记录明细
     * @return 原材料退库记录明细
     */
    @Override
    public List<WmsRawReturnDetail> selectAuditPassRawReturnDetails(WmsRawReturnDetail wmsRawReturnDetail) {
        return wmsRawReturnDetailMapper.selectAuditPassRawReturnDetails(wmsRawReturnDetail);
    }


    /**
     * 更新原材料退库记录的执行状态
     *
     * @param rawReturnId
     * @param rawReturnDetailId
     */
    private void updateRawReturnDetailExecuteStatus(Long rawReturnId, Long rawReturnDetailId) {
        WmsRawReturnDetail queryRawReturnDetail = new WmsRawReturnDetail();
        queryRawReturnDetail.setRawReturnId(rawReturnId);
        List<WmsRawReturnDetail> wmsRawReturnDetails = wmsRawReturnDetailMapper.selectWmsRawReturnDetailList(queryRawReturnDetail);

        boolean isExecuteFinish = true;
        for (WmsRawReturnDetail wmsRawReturnDetail : wmsRawReturnDetails) {
            if (!wmsRawReturnDetail.getRawReturnDetailId().equals(rawReturnDetailId)
                    && !wmsRawReturnDetail.getExecuteStatus().equals(WmsConstants.RAW_RETURN_EXECUTE_STATUS_FINISH)) {
                isExecuteFinish = false;
            }
        }

        if (isExecuteFinish) {
            WmsRawReturn wmsRawReturn = wmsRawReturnMapper.selectWmsRawReturnByRawReturnId(rawReturnId);
            wmsRawReturn.setExecuteStatus(WmsConstants.RAW_OUTSTOCK_EXECUTE_STATUS_FINISH);
            wmsRawReturnMapper.updateWmsRawReturn(wmsRawReturn);
        }
    }

}
